@@ -449,7 +449,7 @@ def _fits_keyword_loader(hdulist, fits_keyword, schema, hdu_index, known_keyword
     return val
 
 
-def _fits_array_loader(hdulist, schema, hdu_index, known_datas):
+def _fits_array_loader(hdulist, schema, hdu_index, known_datas, context):
     hdu_name = _get_hdu_name(schema)
     _assert_non_primary_hdu(hdu_name)
     try:
@@ -458,7 +458,7 @@ def _fits_array_loader(hdulist, schema, hdu_index, known_datas):
         return None
 
     known_datas.add(hdu)
-    return from_fits_hdu(hdu, schema)
+    return from_fits_hdu(hdu, schema, context._cast_fits_arrays)
 
 
 def _schema_has_fits_hdu(schema):
@@ -507,7 +507,7 @@ def _load_from_schema(hdulist, schema, tree, context, skip_fits_update=False):
         elif 'fits_hdu' in schema and (
                 'max_ndim' in schema or 'ndim' in schema or 'datatype' in schema):
             result = _fits_array_loader(
-                hdulist, schema, ctx.get('hdu_index'), known_datas)
+                hdulist, schema, ctx.get('hdu_index'), known_datas, context)
 
             if result is None and context._validate_on_assignment:
                 validate.value_change(path, result, schema, context)
@@ -633,24 +633,31 @@ def from_fits_asdf(hdulist,
                                       ignore_missing_extensions=ignore_missing_extensions)
 
 
-def from_fits_hdu(hdu, schema):
+def from_fits_hdu(hdu, schema, cast_arrays=True):
     """
     Read the data from a fits hdu into a numpy ndarray
     """
     data = hdu.data
 
-    # Save the column listeners for possible restoration
-    if hasattr(data, '_coldefs'):
-        listeners = data._coldefs._listeners
+    if cast_arrays:
+        # Save the column listeners for possible restoration
+        if hasattr(data, '_coldefs'):
+            listeners = data._coldefs._listeners
+        else:
+            listeners = None
+
+        # Cast array to type mentioned in schema
+        data = properties._cast(data, schema)
+
+        # Casting a table loses the column listeners, so restore them
+        if listeners is not None:
+            data._coldefs._listeners = listeners
     else:
-        listeners = None
-
-    # Cast array to type mentioned in schema
-    data = properties._cast(data, schema)
-
-    # Casting a table loses the column listeners, so restore them
-    if listeners is not None:
-        data._coldefs._listeners = listeners
+        # Correct the pseudo-unsigned int problem (this normally occurs
+        # inside properties._cast, but we still need to do it even
+        # when not casting, otherwise arrays from FITS will fail validation).
+        if isinstance(data, fits.FITS_rec):
+            data.dtype = util.rebuild_fits_rec_dtype(data)
 
     return data
 
